#include "Renderer.h"

float Angle = 0.0;
Camera cam = Camera(glm::vec3(16.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.0f), 90.0f);
glm::mat4 projection = glm::perspective(glm::radians(cam.Zoom), (800.0f / 600.0f), 0.001f, 100.0f);

BarinkEngine::Renderer::Renderer()
{
	models = std::vector<GPU_Bucket*>();
}

BarinkEngine::Renderer::~Renderer()
{
	// CleanUp!

	// For each model submitted 
	for ( auto packet : models )
	{
		delete packet;
	}

	// glDeleteBuffers(1, &UV_id);
}


void BarinkEngine::Renderer::Render() 
{
	Angle += 0.0001f;

	for (auto model : models) {
		// Push matrices etc ....
		model->vertexarray.Bind();

		model->elementBuffer.Bind(true);

		if (model->material == nullptr) {
			std::cout << "No material attached!" << std::endl;
			//			continue; //NOTE: Or use some default Material 
		}
		else {
			model->material->shader.Use();
			model->material->Apply();

			model->material->shader.setUniformVec3("Color", model->material->Color);


			model->material->shader.setUniformMat4("M", glm::rotate(glm::mat4(), Angle, glm::vec3(0.5f, 0.5f, 0.0f)));
			model->material->shader.setUniformMat4("V", cam.GetViewMatrix());
			model->material->shader.setUniformMat4("P", projection);
			// Update perf counters 
			ES.verts = model->mesh->vertices.size();
			ES.DC++;

			glDrawElements(GL_TRIANGLES,
				static_cast<unsigned int>(model->mesh->elements.size()),
				GL_UNSIGNED_INT,
				NULL
			);
		}


		model->vertexarray.Unbind();

	}
}


void BarinkEngine::Renderer::Render(Framebuffer& framebuffer)
{
	glBindFramebuffer(GL_FRAMEBUFFER, framebuffer.GetId());
	glClearColor(1.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	Angle += 0.0001f;
	
	for (auto model : models) {
		// Push matrices etc ....
		model->vertexarray.Bind();

		model->elementBuffer.Bind(true);
		
		if (model->material == nullptr) {
			std::cout << "No material attached!" << std::endl;
//			continue; //NOTE: Or use some default Material 
		}
		else {
			model->material->shader.Use();
			model->material->Apply();

			model->material->shader.setUniformVec3("Color", model->material->Color);
			
			
			model->material->shader.setUniformMat4("M", glm::rotate(glm::mat4(), Angle, glm::vec3(0.5f, 0.5f, 0.0f)));
			model->material->shader.setUniformMat4("V", cam.GetViewMatrix());
			model->material->shader.setUniformMat4("P", projection);
			// Update perf counters 
			ES.verts = model->mesh->vertices.size();
			ES.DC++;

			glDrawElements(GL_TRIANGLES,
				static_cast<unsigned int>(model->mesh->elements.size()),
				GL_UNSIGNED_INT,
				NULL
			);
		}
		
	
		model->vertexarray.Unbind();

	}


	glBindFramebuffer(GL_FRAMEBUFFER, 0);
}


// Upload data to cpu and add object to render list
void BarinkEngine::Renderer::Submit(Renderable* model)
{
	// Upload mesh data to gpu for render  
	
	GPU_Bucket* bucket = new GPU_Bucket();

	bucket->material = model->material;
	
	bucket->Upload(*model->mesh);
	
	models.push_back(bucket);
	
}
